package cofh.thermalexpansion.entity.projectile;

import cofh.core.init.CoreProps;
import cofh.core.util.helpers.ServerHelper;
import cofh.thermalexpansion.ThermalExpansion;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.projectile.EntityThrowable;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.registry.EntityRegistry;

public class EntityMorb extends EntityThrowable {

	private static DataParameter<NBTTagCompound> MOB = EntityDataManager.createKey(EntityMorb.class, DataSerializers.COMPOUND_TAG);

	protected static ItemStack blockCheck = new ItemStack(Blocks.STONE);

	protected float gravity = 0.03F;
	protected NBTTagCompound mob;

	public static void initialize(int id) {

		EntityRegistry.registerModEntity(new ResourceLocation("thermalexpansion:morb"), EntityMorb.class, "florb", id, ThermalExpansion.instance, CoreProps.ENTITY_TRACKING_DISTANCE, 10, true);
	}

	/* Required Constructor */
	public EntityMorb(World world) {

		super(world);
	}

	/* Mob Constructors */
	public EntityMorb(World world, EntityLivingBase thrower, NBTTagCompound mob) {

		super(world, thrower);

		if (mob == null || !mob.hasKey("id")) {
			mob = new NBTTagCompound();
		}
		this.mob = mob;

		setGravity();
		setSyncMob();
	}

	public EntityMorb(World world, double x, double y, double z, NBTTagCompound mob) {

		super(world, x, y, z);

		if (mob == null || !mob.hasKey("id")) {
			mob = new NBTTagCompound();
		}
		this.mob = mob;

		setGravity();
		setSyncMob();
	}

	private void setGravity() {

		if (mob.hasNoTags()) {
			this.gravity = 0.02F;
		}
	}

	private void setSyncMob() {

		this.dataManager.set(MOB, mob);
	}

	private NBTTagCompound getSyncMob() {

		return dataManager.get(MOB);
	}

	@Override
	public void onEntityUpdate() {

		if ((mob == null || !mob.hasKey("id")) && ServerHelper.isClientWorld(world)) {
			mob = getSyncMob();
		}
		super.onEntityUpdate();
	}

	@Override
	protected void onImpact(RayTraceResult traceResult) {

		BlockPos pos = traceResult.getBlockPos();

	}

	@Override
	protected void entityInit() {

		dataManager.register(MOB, new NBTTagCompound());
	}

	@Override
	protected float getGravityVelocity() {

		return gravity;
	}

	public NBTTagCompound getMob() {

		return mob;
	}

	/* NBT METHODS */
	@Override
	public void readEntityFromNBT(NBTTagCompound nbt) {

		super.readEntityFromNBT(nbt);

		mob = nbt.getCompoundTag("Mob");
	}

	@Override
	public void writeEntityToNBT(NBTTagCompound nbt) {

		super.writeEntityToNBT(nbt);

		nbt.setTag("Mob", mob);
	}

}
